//
//  ContentView.swift
//  CreatewithSwift-LiveActivities
//
//  Created by Matteo Altobello
//

import SwiftUI

struct ContentView: View {

    // 1.
    @State private var isPrinting = false
    @State private var timer: Timer? = nil
    @State var viewModel = PrintingActivityViewModel()
    
    // 2.
    var body: some View {
        VStack(spacing: 30) {
            Image(systemName: "printer.fill.and.paper.fill")
                .font(.system(size: 80))
                .foregroundColor(isPrinting ? .blue : .gray)
            
            Text(isPrinting ? "Printing in progress..." : "Ready to print")
                .font(.title)
                .fontWeight(.bold)
            
            if isPrinting {
                VStack(spacing: 15) {
                    Text("Printing: \(viewModel.printName)")
                        .font(.headline)
                    
                    HStack {
                        Text(String(format: "%02d:%02d", Int(viewModel.elapsedTime) / 60, Int(viewModel.elapsedTime) % 60))
                        Text("/")
                        Text(String(format: "%02d:%02d", Int(viewModel.printDuration) / 60, Int(viewModel.printDuration) % 60))
                    }

                    
                    Gauge(value: viewModel.progress) {
                        Text("Progress")
                    } currentValueLabel: {
                        Text("\(Int(viewModel.progress * 100))%")
                    }
                    .gaugeStyle(.accessoryLinear)
                    .tint(.blue)
                }
                .padding()
                .background(RoundedRectangle(cornerRadius: 12).fill(Color(.systemBackground)))
                .shadow(radius: 3)
            }
            
            Button(action: isPrinting ? cancelPrint : startPrint) {
                Text(isPrinting ? "Cancel Print" : "Start Print")
                    .font(.headline)
                    .foregroundColor(.white)
                    .frame(width: 200, height: 50)
                    .background(isPrinting ? Color.red : Color.blue)
                    .cornerRadius(10)
            }
        }
        .padding()
        .onDisappear {
            timer?.invalidate()
        }
    }


    // 3.
    func startPrint() {
        isPrinting = true

        timer = Timer.scheduledTimer(withTimeInterval: 1.0, repeats: true) { _ in
            viewModel.elapsedTime += 1
            viewModel.progress = min(viewModel.elapsedTime / viewModel.printDuration, 1.0)
            viewModel.updateLiveActivity()
            
            // End print when complete
            if viewModel.elapsedTime >= viewModel.printDuration {
                finishPrint()
            }
        }
        
        // Start Live Activity
        viewModel.startLiveActivity()
    }
    
    func cancelPrint() {
        timer?.invalidate()
        timer = nil
        isPrinting = false
        
        // End Live Activity
        viewModel.endLiveActivity()
    }
    
    func finishPrint() {
        timer?.invalidate()
        timer = nil
        isPrinting = false
        
        // End Live Activity with success
        viewModel.endLiveActivity(success: true)
    }
}

#Preview {
    ContentView()
}
